MissionMinder = MissionMinder or {};
MissionMinder.ptr = GetBuildInfo():match('6[.]1[.][0-9]+')

local addon = MissionMinder;
addon.name = "MissionMinder"; --internal name; purposefully non-localized
addon.version = GetAddOnMetadata(addon.name, "Version");
addon.options = {};
local options = addon.options;

local PreLoadLib = MissionMinderPreLoad

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local L_CONFIGURE = L["Configure MissionMinder"]
local L_ADDON_DESCRIPTION = L["Manage garrison missions in a more streamline, less-invasive manner"]
local L_LISTVIEW = L["List View"]
local L_LISTVIEW_DESCRIPTION = L["View completed missions as a list from the \"In Progress\" tab"]
local L_ENABLED = L["Enabled"]
local L_LISTVIEW_ENABLED_DESCRIPTION = L["View completed missions as a list from the \"In Progress\" tab"]
local L_QUICKLOOT = L["Quick loot"]
local L_QUICKLOOT_DESCRIPTION = L["Disable the completion frame entirely, skip all the fanfare, and just give me the loot already!"]
local L_SHOWXP = L["Show follower XP gains"]
local L_SHOWXP_DESCRIPTION = L["Show messages regarding individual follower XP gains"]
local L_GENERAL_OPTIONS = L["General Options"]
local L_SKIPFIGHTANIMATIONS = L["Skip fight animations"]
local L_SKIPFIGHTANIMATIONS_DESCRIPTION = L["Skip all fight animations and go straight to rewards"]
local L_AUTOOPENREWARDS = L["Automatically open rewards chest"]
local L_AUTOOPENREWARDS_DESCRIPTION = L["Remove the need to click on the rewards chest to recieve the rewards"]
local L_AUTOCLOSE = L["Automatically close on loot"]
local L_AUTOCLOSE_DESCRIPTION = L["Automatically close the completion dialog if all actions have been completed for a mission"]
local L_PLANNER = L["Planner"];
local L_SHOWFOLLOWERCOUNTERTOTALS = L["Show active follower counter totals"];
local L_SHOWFOLLOWERCOUNTERTOTALS_DESCRIPTION = L["Easily track how many of each counter you currently have active"]
local L_IGNOREXPGAINTRAITS = L["Ignore XP increase traits"]
local L_IGNOREXPGAINTRAITS_DESCRIPTION = L["Do not show XP increase traits on missions that do not reward Bonus XP"]

function MissionMinder.debug(format, ...)
	--[===[@alpha@
	print("|cffcccccc[DEBUG] " .. string.format(format, ...));
	--@end-alpha@]===]
end

local DEBUG = MissionMinder.debug;

---------------------------------------------
-- OPTIONS
---------------------------------------------

local LISTVIEW_ENABLED = "ListViewEnabled"
local AUTO_OPEN_REWARDS = "AutoOpenRewards";
local AUTO_CLOSE = "AutoClose";
local SKIP_FIGHT_ANIMATIONS = "SkipFightAnimations";
local QUICK_LOOT = "QuickLoot";
local SHOW_FOLLOWER_XP_GAINS = "ShowFollowerXPGains";
local SHOW_FOLLOWER_COUNTER_TOTALS = "ShowFollowerCounterTotals";
local IGNORE_XPGAINS_TRAITS = "IgnoreXPGainTraits";

local INPROGRESS_MISSIONS = "InProgressMissions";


MissionMinderOptions = {};
local MissionMinderOptions_DEFAULT = {
	[LISTVIEW_ENABLED] = true,
	[AUTO_OPEN_REWARDS] = false,
	[AUTO_CLOSE] = false,
	[SKIP_FIGHT_ANIMATIONS] = false,
	[QUICK_LOOT] = false,
	[SHOW_FOLLOWER_XP_GAINS] = false,
	[SHOW_FOLLOWER_COUNTER_TOTALS] = true,
	[IGNORE_XPGAINS_TRAITS] = true
};

function options.ListViewEnabled()
	return MissionMinderOptions[LISTVIEW_ENABLED];
end

function options.AutoLootRewards()
	return MissionMinderOptions[AUTO_OPEN_REWARDS];
end

function options.AutoClose()
	return MissionMinderOptions[AUTO_CLOSE];
end

function options.SkipFightAnimations()
	return MissionMinderOptions[SKIP_FIGHT_ANIMATIONS];
end

function options.QuickLoot()
	return options.ListViewEnabled() and MissionMinderOptions[QUICK_LOOT];
end

function options.ShowFollowerXPGains()
	return options.QuickLoot() and MissionMinderOptions[SHOW_FOLLOWER_XP_GAINS];
end

function options.ShowFollowerCounterTotals()
	return MissionMinderOptions[SHOW_FOLLOWER_COUNTER_TOTALS];
end

function options.IgnoreXPGainTraits()
	return MissionMinderOptions[IGNORE_XPGAINS_TRAITS];
end


---------------------------------------------
-- OPTIONS PANE
---------------------------------------------

local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

local function createOptions()
	local function getValue(info)
		return MissionMinderOptions[info.arg];
	end

	local function setValue(info, value)
		MissionMinderOptions[info.arg] = value;
	end

	local options = {
		name = "MissionMinder " .. addon.version,
		desc = L_ADDON_DESCRIPTION,
		type = "group",
		args = {
			desc = {
				order = 2,
				type = "description",
				name = L_ADDON_DESCRIPTION,
				width = "full"
			},
			emptySpace = {
				order = 3,
				type = "description",
				name = " ",
				width = "full"
			},
			listview = {
				order = 10,
				type = "group",
				inline = true,
				name = L_LISTVIEW,
				args = {
					description = {
						order = 1,
						type = "description",
						name = L_LISTVIEW_DESCRIPTION,
						width = "full",
					},
					spacer = {
						order = 2,
						type = "description",
						name = " ",
						width = "full",
					},
					listviewEnabled = {
						order = 10,
						name = L_ENABLED,
						desc = L_LISTVIEW_ENABLED_DESCRIPTION,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = LISTVIEW_ENABLED
					},
					quickLoot = {
						order = 25,
						name = L_QUICKLOOT,
						desc = L_QUICKLOOT_DESCRIPTION,
						disabled = function() return not options.ListViewEnabled(); end,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = QUICK_LOOT
					},
					showXP = {
						order = 30,
						name = L_SHOWXP,
						desc = L_SHOWXP_DESCRIPTION,
						disabled = function() return not options.ListViewEnabled() or not options.QuickLoot(); end,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = SHOW_FOLLOWER_XP_GAINS
					}
				}
			},
			general = {
				order = 15,
				type = "group",
				inline = true,
				disabled = options.QuickLoot,
				name = L_GENERAL_OPTIONS,
				args = {
					skipFightAnimations = {
						order = 5,
						name = L_SKIPFIGHTANIMATIONS,
						desc = L_SKIPFIGHTANIMATIONS_DESCRIPTION,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = SKIP_FIGHT_ANIMATIONS
					},
					autoOpenRewards = {
						order = 10,
						name = L_AUTOOPENREWARDS,
						desc = L_AUTOOPENREWARDS_DESCRIPTION,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = AUTO_OPEN_REWARDS
					},
					autoClose = {
						order = 15,
						name = L_AUTOCLOSE,
						desc = L_AUTOCLOSE_DESCRIPTION,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = AUTO_CLOSE
					}
				}
			},
			planner = {
				order = 20,
				type = "group",
				inline = true,
				name = L_PLANNER,
				args = {
					showFollowerCounterTotals = {
						order = 5,
						name = L_SHOWFOLLOWERCOUNTERTOTALS,
						desc = L_SHOWFOLLOWERCOUNTERTOTALS_DESCRIPTION,
						type = "toggle",
						width = "full",
						set = function(info, value)
							setValue(info, value);
							if ( GarrisonMissionFrame:IsShown() and GarrisonMissionFrame.FollowerTab:IsShown()) then
								if(value and not MissionMinderActiveFollowerCounters:IsShown() ) then
									MissionMinderActiveFollowerCounters:Show();
									if ( MissionMinder.ptr ) then
										GarrisonThreatCountersFrame:Hide();
									end
								elseif(not value and MissionMinderActiveFollowerCounters:IsShown()) then
									MissionMinderActiveFollowerCounters:Hide();
									if ( MissionMinder.ptr ) then
										GarrisonThreatCountersFrame:Show();
									end
								end
							end
						end,
						get = getValue,
						arg = SHOW_FOLLOWER_COUNTER_TOTALS
					},
					ignoreXPGainTraits = {
						order = 10,
						name = L_IGNOREXPGAINTRAITS,
						desc = L_IGNOREXPGAINTRAITS_DESCRIPTION,
						type = "toggle",
						width = "full",
						set = setValue,
						get = getValue,
						arg = IGNORE_XPGAINS_TRAITS
					}
				}
			}
		}
	};

	config:RegisterOptionsTable("MissionMinderOptions", options);
	return dialog:AddToBlizOptions("MissionMinderOptions", "MissionMinder");
end

local function ShowConfigureTooltip(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(L_CONFIGURE);
end

local frame = createOptions();
addon.options.frame = frame;


function frame.OnEvent(self, event, arg1)
	if event == "ADDON_LOADED" and arg1 == addon.name then
		frame:UnregisterEvent("ADDON_LOADED");

		-- SavedVariables have been loaded, create new values
		for key,value in pairs(MissionMinderOptions_DEFAULT) do
			if not MissionMinderOptions[key] then
				MissionMinderOptions[key] = value;
			end
		end

		-- Remove any deprecated settings
		for key,value in pairs(MissionMinderOptions) do
			if not MissionMinderOptions_DEFAULT[key] then
				MissionMinderOptions[key] = nil;
			end
		end


		GarrisonMissionFrameConfigureButton:HookScript("OnEnter", ShowConfigureTooltip)
		GarrisonLandingPageConfigureButton:HookScript("OnEnter", ShowConfigureTooltip)

		-- Copy MissionMinderGlobals to MissionMinderPreLoadGlobals
		if(MissionMinderGlobals) then
			for character, inprogressMissions in pairs(MissionMinderGlobals.InProgressMissions) do
				local missions = PreLoadLib.Globals.Characters[character].missions
				if ( not missions ) then
					missions = {}
					PreLoadLib.Globals.Characters[character].missions = missions
				end

				for missionID, endDate in pairs(inprogressMissions) do
					if ( not missions[missionID] ) then
						missions[missionID] = endDate
					end
				end
			end
		end
 	end
end
frame:SetScript("OnEvent", frame.OnEvent);
frame:RegisterEvent("ADDON_LOADED");
